==> ibm_watsonx_ai-0.2.3-py3-none-any.whl: ibm_watsonx_ai/workspace/workspace.py <==
#  -----------------------------------------------------------------------------------------
#  (C) Copyright IBM Corp. 2023-2024.
#  https://opensource.org/licenses/BSD-3-Clause
#  -----------------------------------------------------------------------------------------

from warnings import warn

from ibm_watsonx_ai import APIClient
from ibm_watsonx_ai.wml_client_error import MissingArgument, WrongEnvironmentVersion
from ibm_watsonx_ai.wml_client_error import WMLClientError
from ..utils.autoai.enums import TShirtSize
from ..utils.autoai.errors import TShirtSizeNotSupported, SetIDFailed
from ..utils.autoai.utils import is_ipython


class WorkSpace:
    """WorkSpace class for Service authentication and project/space manipulation."""

    def __init__(self,
                 credentials: dict = None,
                 project_id: str = None,
                 space_id: str = None,
                 verify=None,
                 **kwargs) -> None:
        """
        :param credentials: credentials to Service instance
        :type credentials: dict

        :param project_id: ID of the Watson Studio project
        :type project_id: str, optional

        :param space_id: ID of the Watson Studio Space
        :type space_id: str, optional

        **Example**

        .. code-block: python

            from ibm_watsonx_ai.workspace import WorkSpace

            ws = WorkSpace(
                credentials={
                    "apikey": "...",
                    "iam_apikey_description": "...",
                    "iam_apikey_name": "...",
                    "iam_role_crn": "...",
                    "iam_serviceid_crn": "...",
                    "instance_id": "...",
                    "url": "https://us-south.ml.cloud.ibm.com"
                },
                project_id="...",
                space_id="...")
        """
        # note: backward compatibility
        if (wml_credentials:=kwargs.get('wml_credentials')) is not None:
            credentials = wml_credentials
            warn(("`wml_credentials` is deprecated and will be removed in future. "
                  "Instead, please use `credentials`."), category=DeprecationWarning)
        if wml_credentials is None and credentials is None:
            raise WMLClientError("No `credentials` provided")
        # --- end note
        self.credentials = credentials.copy()
        self.project_id = project_id
        self.space_id = space_id

        self.api_client = APIClient(wml_credentials=self.credentials, verify=verify)

        if not credentials.get('instance_id'):
            if self.space_id is not None:
                self.api_client.set.default_space(self.space_id)

            elif self.project_id is not None:
                self.api_client.set.default_project(self.project_id)

            else:
                raise SetIDFailed(
                    f'project_id and space_id',
                    reason=f'project_id and space_id cannot be None at the same time.')

        elif credentials.get('instance_id', '').lower() in ('icp', 'openshift'):
            if self.project_id is None and self.space_id is None:
                raise MissingArgument(
                    'project_id',
                    reason="These credentials are from CP4D environment, "
                           "please specify \"project_id\"")
            else:
                if self.project_id is not None:
                    outcome = self.api_client.set.default_project(self.project_id)

                else:
                    outcome = self.api_client.set.default_space(self.space_id)

                if outcome == 'FAILURE':
                    raise SetIDFailed(
                        f'{"project_id" if self.project_id is not None else "space_id"}',
                        reason=f'This {"project_id" if self.project_id is not None else "space_id"}: '
                               f'{self.project_id if self.project_id is not None else self.space_id} '
                               f'cannot be found in current environment.')

            supported_versions = ('3.0.0', '3.0.1', '3.5', '4.0', '4.5', '4.6', '4.7', '4.8')
            if credentials[u'version'] not in supported_versions:
                raise WrongEnvironmentVersion(credentials[u'version'], credentials[u'instance_id'].lower(),
                                              supported_versions)

    def __str__(self):
        return f'credentials: {self.credentials} project_id: {self.project_id} space_id = {self.space_id}'

    def __repr__(self):
        return self.__str__()
    
    @property
    def wml_credentials(self):
        warn(("`wml_credentials` is deprecated and will be removed in future. "
              "Instead, please use `credentials`."), category=DeprecationWarning)
        return self.credentials
    
    @wml_credentials.setter
    def wml_credentials(self, var):
        warn(("`wml_credentials` is deprecated and will be removed in future. "
              "Instead, please use `credentials`."), category=DeprecationWarning)
        self.credentials = var

    @property
    def wml_client(self):
        warn(("`wml_client` is deprecated and will be removed in future. "
              "Instead, please use `api_client`."), category=DeprecationWarning)
        return self.api_client
    
    @wml_client.setter
    def wml_client(self, var):
        warn(("`wml_client` is deprecated and will be removed in future. "
              "Instead, please use `api_client`."), category=DeprecationWarning)
        self.api_client = var


    def restrict_pod_size(self, t_shirt_size: 'TShirtSize') -> 'TShirtSize':
        """Check t_shirt_size for AutoAI POD. Restrict sizes per environment.

        :param t_shirt_size: TShirt size to be validated and restricted
        :type t_shirt_size: TShirtSize

        :return: validated and restricted TShirt size
        :rtype: TShirtSize
        """
        # note: for testing purposes
        if self.credentials.get('development', False):
            return t_shirt_size
        # --- end note

        default_cloud = TShirtSize.L
        default_cp4d = TShirtSize.M
        default_server = TShirtSize.M

        supported_cp4d = (TShirtSize.S, TShirtSize.M, TShirtSize.L, TShirtSize.XL)
        supported_server = (TShirtSize.S, TShirtSize.M, TShirtSize.L)

        # note: check CP4D and Server pod sizes
        if self.api_client.ICP:
            if t_shirt_size not in supported_cp4d:
                message = (f"This t-shirt-size: \"{t_shirt_size}\" is not supported in CP4D. "
                           f"Supported sizes: {supported_cp4d} "
                           f"Continuing work with default size {default_cp4d}")

                warn(message=message)
                if is_ipython():
                    print(message)

                return default_cp4d

            else:
                return t_shirt_size

        else:
            # note: allow every size in test envs
            if 'test' in self.credentials.get('url', ''):
                return t_shirt_size

            else:
                # note: raise an error for cloud if pod size is different, other just return
                if t_shirt_size != default_cloud:
                    raise TShirtSizeNotSupported(
                        t_shirt_size,
                        reason=f"This t-shirt size is not supported. Please use the default one: {default_cloud}")

                else:
                    return default_cloud
                # --- end note

==> ibm_watsonx_ai-0.2.6-py3-none-any.whl: ibm_watsonx_ai/workspace/workspace.py <==
#  -----------------------------------------------------------------------------------------
#  (C) Copyright IBM Corp. 2023-2024.
#  https://opensource.org/licenses/BSD-3-Clause
#  -----------------------------------------------------------------------------------------

from warnings import warn

from ibm_watsonx_ai import APIClient
from ibm_watsonx_ai.wml_client_error import MissingArgument, WrongEnvironmentVersion
from ibm_watsonx_ai.wml_client_error import WMLClientError
from ..utils.autoai.enums import TShirtSize
from ..utils.autoai.errors import TShirtSizeNotSupported, SetIDFailed
from ..utils.autoai.utils import is_ipython


class WorkSpace:
    """WorkSpace class for Service authentication and project/space manipulation."""

    def __init__(self,
                 credentials: dict = None,
                 project_id: str = None,
                 space_id: str = None,
                 verify=None,
                 **kwargs) -> None:
        """
        :param credentials: credentials to Service instance
        :type credentials: dict

        :param project_id: ID of the Watson Studio project
        :type project_id: str, optional

        :param space_id: ID of the Watson Studio Space
        :type space_id: str, optional

        **Example**

        .. code-block: python

            from ibm_watsonx_ai.workspace import WorkSpace

            ws = WorkSpace(
                credentials={
                    "apikey": "...",
                    "iam_apikey_description": "...",
                    "iam_apikey_name": "...",
                    "iam_role_crn": "...",
                    "iam_serviceid_crn": "...",
                    "instance_id": "...",
                    "url": "https://us-south.ml.cloud.ibm.com"
                },
                project_id="...",
                space_id="...")
        """
        # note: backward compatibility
        if (wml_credentials:=kwargs.get('wml_credentials')) is not None:
            credentials = wml_credentials
            warn(("`wml_credentials` is deprecated and will be removed in future. "
                  "Instead, please use `credentials`."), category=DeprecationWarning)
        if wml_credentials is None and credentials is None:
            raise WMLClientError("No `credentials` provided")
        # --- end note
        self.credentials = credentials.copy()
        self.project_id = project_id
        self.space_id = space_id

        self.api_client = APIClient(wml_credentials=self.credentials, verify=verify)

        if not credentials.get('instance_id'):
            if self.space_id is not None:
                self.api_client.set.default_space(self.space_id)

            elif self.project_id is not None:
                self.api_client.set.default_project(self.project_id)

            else:
                raise SetIDFailed(
                    f'project_id and space_id',
                    reason=f'project_id and space_id cannot be None at the same time.')

        elif credentials.get('instance_id', '').lower() in ('icp', 'openshift'):
            if self.project_id is None and self.space_id is None:
                raise MissingArgument(
                    'project_id',
                    reason="These credentials are from CP4D environment, "
                           "please specify \"project_id\"")
            else:
                if self.project_id is not None:
                    outcome = self.api_client.set.default_project(self.project_id)

                else:
                    outcome = self.api_client.set.default_space(self.space_id)

                if outcome == 'FAILURE':
                    raise SetIDFailed(
                        f'{"project_id" if self.project_id is not None else "space_id"}',
                        reason=f'This {"project_id" if self.project_id is not None else "space_id"}: '
                               f'{self.project_id if self.project_id is not None else self.space_id} '
                               f'cannot be found in current environment.')

            supported_versions = ('4.0', '4.5', '4.6', '4.7', '4.8', '5.0')
            if credentials[u'version'] not in supported_versions:
                raise WrongEnvironmentVersion(credentials[u'version'], credentials[u'instance_id'].lower(),
                                              supported_versions)

    def __str__(self):
        return f'credentials: {self.credentials} project_id: {self.project_id} space_id = {self.space_id}'

    def __repr__(self):
        return self.__str__()
    
    @property
    def wml_credentials(self):
        warn(("`wml_credentials` is deprecated and will be removed in future. "
              "Instead, please use `credentials`."), category=DeprecationWarning)
        return self.credentials
    
    @wml_credentials.setter
    def wml_credentials(self, var):
        warn(("`wml_credentials` is deprecated and will be removed in future. "
              "Instead, please use `credentials`."), category=DeprecationWarning)
        self.credentials = var

    @property
    def wml_client(self):
        warn(("`wml_client` is deprecated and will be removed in future. "
              "Instead, please use `api_client`."), category=DeprecationWarning)
        return self.api_client
    
    @wml_client.setter
    def wml_client(self, var):
        warn(("`wml_client` is deprecated and will be removed in future. "
              "Instead, please use `api_client`."), category=DeprecationWarning)
        self.api_client = var


    def restrict_pod_size(self, t_shirt_size: 'TShirtSize') -> 'TShirtSize':
        """Check t_shirt_size for AutoAI POD. Restrict sizes per environment.

        :param t_shirt_size: TShirt size to be validated and restricted
        :type t_shirt_size: TShirtSize

        :return: validated and restricted TShirt size
        :rtype: TShirtSize
        """
        # note: for testing purposes
        if self.credentials.get('development', False):
            return t_shirt_size
        # --- end note

        default_cloud = TShirtSize.L
        default_cp4d = TShirtSize.M
        default_server = TShirtSize.M

        supported_cp4d = (TShirtSize.S, TShirtSize.M, TShirtSize.L, TShirtSize.XL)
        supported_server = (TShirtSize.S, TShirtSize.M, TShirtSize.L)

        # note: check CP4D and Server pod sizes
        if self.api_client.ICP:
            if t_shirt_size not in supported_cp4d:
                message = (f"This t-shirt-size: \"{t_shirt_size}\" is not supported in CP4D. "
                           f"Supported sizes: {supported_cp4d} "
                           f"Continuing work with default size {default_cp4d}")

                warn(message=message)
                if is_ipython():
                    print(message)

                return default_cp4d

            else:
                return t_shirt_size

        else:
            # note: allow every size in test envs
            if 'test' in self.credentials.get('url', ''):
                return t_shirt_size

            else:
                # note: raise an error for cloud if pod size is different, other just return
                if t_shirt_size != default_cloud:
                    raise TShirtSizeNotSupported(
                        t_shirt_size,
                        reason=f"This t-shirt size is not supported. Please use the default one: {default_cloud}")

                else:
                    return default_cloud
                # --- end note
